#!/usr/bin/env julia
#
# @license Apache-2.0
#
# Copyright (c) 2018 The Stdlib Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import JSON

"""
    gen( domain, name )

Generate fixture data and write to file.

# Arguments

* `domain`: domain
* `name::AbstractString`: output filename

# Examples

``` julia
julia> x = range( -1000.0, stop = 1000.0, length = 2001 );
julia> gen( x, \"data.json\" );
```
"""
function gen( domain, name )
	x = collect( domain );
	y = tan.( x );

	# Store data to be written to file as a collection:
	data = Dict([
		("x", x),
		("expected", y)
	]);

	# Based on the script directory, create an output filepath:
	filepath = joinpath( dir, name );

	# Write the data to the output filepath as JSON:
	outfile = open( filepath, "w" );
	write( outfile, JSON.json(data) );
	close( outfile );
end

# Get the filename:
file = @__FILE__;

# Extract the directory in which this file resides:
dir = dirname( file );

# Subnormal values:
x = range( 1.0e-309, stop = 1.0e-324, length = 500 );
gen( x, "subnormal.json" );

# Negative tiny values:
x = range( -1.0e-300, stop = -1.0e-308, length = 500 );
gen( x, "tiny_negative.json" );

# Positive tiny values:
x = range( 1.0e-300, stop = 1.0e-308, length = 500 );
gen( x, "tiny_positive.json" );

# Small(er) values:
x = range( -pi, stop = pi, length = 2000 );
gen( x, "smaller.json" );

# Negative small values:
x = range( -4.0*pi, stop = -pi, length = 1000 );
gen( x, "small_negative.json" );

# Positive small values:
x = range( pi, stop = 4.0*pi, length = 1000 );
gen( x, "small_positive.json" );

# Negative medium values:
x = range( -4.0*pi, stop = 12.0*pi, length = 1000 );
gen( x, "medium_negative.json" );

# Positive medium values:
x = range( 4.0*pi, stop = 12.0*pi, length = 1000 );
gen( x, "medium_positive.json" );

# Large negative values:
x = range( -12.0*pi, stop = -256.0*pi, length = 1000 );
gen( x, "large_negative.json" );

# Large positive values:
x = range( 12.0*pi, stop = 256.0*pi, length = 1000 );
gen( x, "large_positive.json" );

# Very large negative values:
x = range( -2.0^20*(pi/2.0), stop = -2.0^60*(pi/2.0), length = 500 );
gen( x, "very_large_negative.json" );

# Very large positive values:
x = range( 2.0^20*(pi/2.0), stop = 2.0^60*(pi/2.0), length = 500 );
gen( x, "very_large_positive.json" );

# Huge negative values:
x = range( -2.0^60*(pi/2.0), stop = -2.0^1000*(pi/2.0), length = 1000 )
gen( x, "huge_negative.json" );

# Huge positive values:
x = range( 2.0^20*(pi/2.0), stop = 2.0^60*(pi/2.0), length = 1000 )
gen( x, "huge_positive.json" );
